@@ -38,6 +38,7 @@
 #endif // KDE 3.x
 
 #include <kkeydialog.h>
+#include <kinputdialog.h>
 #include <kiconloader.h>
 #include <kmessagebox.h>
 #include <kfiledialog.h>
@@ -53,6 +54,7 @@
 #include <ktempfile.h>
 #include <kpopupmenu.h>
 #include <kedittoolbar.h>
+#include <kwin.h>
 
 #if KDE_VERSION < 220
 #include <qprinter.h>
@@ -73,7 +75,7 @@
 #endif
 
 KmagApp::KmagApp(QWidget* , const char* name)
-  : KMainWindow(0, name, WStyle_MinMax | WType_TopLevel | WDestructiveClose | WStyle_ContextHelp | WStyle_StaysOnTop),
+  : KMainWindow(0, name, WStyle_MinMax | WType_TopLevel | WStyle_StaysOnTop | WDestructiveClose | WStyle_ContextHelp),
     m_defaultMouseCursorType(2)
 {
   config=kapp->config();
@@ -107,11 +109,11 @@ KmagApp::KmagApp(QWidget* , const char* name)
   initActions();
   initConnections();
 
-  // read options from config file
-  readOptions();
-
   // Register all KIO image formats - to be used when saving image.
   KImageIO::registerFormats();
+
+  // read options from config file
+  readOptions();
 }
 
 /**
@@ -174,17 +176,33 @@ void KmagApp::initActions()
 
   m_modeFollowMouse = new KRadioAction(i18n("&Follow Mouse Mode"), "followmouse", Key_F1, this,
                             SLOT(slotModeFollowMouse()), actionCollection(), "mode_followmouse");
-  m_modeFollowMouse->setToolTip(i18n("Magnify around the mouse cursor"));
-  m_modeFollowMouse->setWhatsThis(i18n("If selected, the area around the mouse cursor is magnified"));
+  m_modeFollowMouse->setToolTip(i18n("Magnify mouse area into window"));
+  m_modeFollowMouse->setWhatsThis(i18n("In this mode the area around the mouse cursor is shown in a normal window."));
 
-  m_modeSelWin = new KRadioAction(i18n("Se&lection Window Mode"), "window", Key_F2, this,
+  m_modeSelWin = new KRadioAction(i18n("S&election Window Mode"), "window", Key_F2, this,
                             SLOT(slotModeSelWin()), actionCollection(), "mode_selectionwindow");
-  m_modeSelWin->setToolTip(i18n("Show a window for selecting the magnified area"));
+  m_modeSelWin->setToolTip(i18n("Magnify selected area into window"));
+  m_modeSelWin->setWhatsThis(i18n("In this mode a selection window is opened. The selected area is shown in a normal window."));
 
-  m_modeWholeScreen = new KRadioAction(i18n("&Whole Screen Mode"), "window_fullscreen", Key_F3, this,
-                              SLOT(slotModeWholeScreen()), actionCollection(),"mode_wholescreen");
-  m_modeWholeScreen->setToolTip(i18n("Magnify the whole screen"));
-  m_modeWholeScreen->setWhatsThis(i18n("Click on this button to fit the zoom view to the zoom window."));
+  m_modeEdgeTop = new KRadioAction(i18n("&Top Screen Edge Mode"), 0, 0, this,
+                              SLOT(slotModeEdgeTop()), actionCollection(),"mode_edgetop");
+  m_modeEdgeTop->setToolTip(i18n("Magnify mouse area to top screen edge"));
+  m_modeEdgeTop->setWhatsThis(i18n("In this mode the area around the mouse is magnified to the top screen edge."));
+
+  m_modeEdgeLeft = new KRadioAction(i18n("&Left Screen Edge Mode"), 0, 0, this,
+                              SLOT(slotModeEdgeLeft()), actionCollection(),"mode_edgeleft");
+  m_modeEdgeLeft->setToolTip(i18n("Magnify mouse area to left screen edge"));
+  m_modeEdgeLeft->setWhatsThis(i18n("In this mode the area around the mouse is magnified to the left screen edge."));
+
+  m_modeEdgeRight = new KRadioAction(i18n("&Right Screen Edge Mode"), 0, 0, this,
+                              SLOT(slotModeEdgeRight()), actionCollection(),"mode_edgeright");
+  m_modeEdgeRight->setToolTip(i18n("Magnify mouse area to right screen edge"));
+  m_modeEdgeRight->setWhatsThis(i18n("In this mode the area around the mouse is magnified to the right screen edge."));
+
+  m_modeEdgeBottom = new KRadioAction(i18n("&Bottom Screen Edge Mode"), 0, 0, this,
+                              SLOT(slotModeEdgeBottom()), actionCollection(),"mode_edgebottom");
+  m_modeEdgeBottom->setToolTip(i18n("Magnify mouse area to bottom screen edge"));
+  m_modeEdgeBottom->setWhatsThis(i18n("In this mode the area around the mouse is magnified to the bottom screen edge."));
 
   m_hideCursor = new KToggleAction(i18n("Hide Mouse &Cursor"), "hidemouse", Key_F4, this,
                             SLOT(slotToggleHideCursor()), actionCollection(), "hidecursor");
@@ -204,6 +222,9 @@ void KmagApp::initActions()
   m_pZoomOut = KStdAction::zoomOut(this, SLOT(zoomOut()), actionCollection(), "zoom_out");
   m_pZoomOut->setWhatsThis(i18n("Click on this button to <b>zoom-out</b> on the selected region."));
 
+  m_pInvert = new KToggleAction(i18n("&Invert Colors"), 0, Key_F6, this,
+                            SLOT(slotToggleInvert()), actionCollection(), "invert");
+
   m_pRotationBox = new KSelectAction(i18n("&Rotation"),0,actionCollection(),"rotation");
   m_pRotationBox->setItems(rotationArrayString);
   m_pRotationBox->setWhatsThis(i18n("Select the rotation degree."));
@@ -215,7 +236,7 @@ void KmagApp::initActions()
   m_toolConf = KStdAction::configureToolbars( this, SLOT( slotEditToolbars() ),
                                               actionCollection(), "toolbar_conf");
 
-  m_pFPSBox = new KSelectAction(i18n("&Refresh"),0,actionCollection(),"fps_selector");
+  m_pFPSBox = new KSelectAction(i18n("Re&fresh"),0,actionCollection(),"fps_selector");
   m_pFPSBox->setItems(fpsArrayString);
   m_pFPSBox->setWhatsThis(i18n("Select the refresh rate. The higher the rate, the more computing power (CPU) will be needed."));
   m_pFPSBox->setToolTip(i18n("Refresh rate"));
@@ -252,6 +273,8 @@ void KmagApp::initConnections()
   connect(m_pZoomBox, SIGNAL(activated(int)), this, SLOT(setZoomIndex(int)));
   connect(m_pRotationBox, SIGNAL(activated(int)), this, SLOT(setRotationIndex(int)));
   connect(m_pFPSBox, SIGNAL(activated(int)), this, SLOT(setFPSIndex(int)));
+
+  connect(m_zoomView, SIGNAL(contextMenu(QPoint)), this, SLOT(contextMenu(QPoint)));
 }
 
 /**
@@ -262,6 +285,7 @@ void KmagApp::saveOptions()
   config->setGroup("General Options");
   config->writeEntry("Geometry", size());
   config->writeEntry("ZoomIndex", m_zoomIndex);
+  config->writeEntry("Invertation", m_pInvert->isChecked());
   config->writeEntry("RotationIndex", m_rotationIndex);
   config->writeEntry("FPSIndex", m_fpsIndex);
   config->writeEntry("SelRect", m_zoomView->getSelRectPos());
@@ -269,10 +293,18 @@ void KmagApp::saveOptions()
 
   if (m_modeFollowMouse->isChecked())
      config->writeEntry("Mode", "followmouse");
-  else if (m_modeWholeScreen->isChecked())
-     config->writeEntry("Mode", "wholescreen");
   else if (m_modeSelWin->isChecked())
      config->writeEntry("Mode", "selectionwindow");
+  else if (m_modeEdgeTop->isChecked())
+     config->writeEntry("Mode", "edgetop");
+  else if (m_modeEdgeLeft->isChecked())
+     config->writeEntry("Mode", "edgeleft");
+  else if (m_modeEdgeRight->isChecked())
+     config->writeEntry("Mode", "edgeright");
+  else if (m_modeEdgeBottom->isChecked())
+     config->writeEntry("Mode", "edgebottom");
+
+  config->writeEntry ("EdgeSize", edgesize);
 
   config->writeEntry("ShowMenu", m_pShowMenu->isChecked());
   config->writeEntry("ShowMainToolBar", m_pShowMainToolBar->isChecked());
@@ -314,6 +346,9 @@ void KmagApp::readOptions()
   setZoomIndex(zoomIndex);
   emit updateZoomIndex(m_zoomIndex);
 
+  m_pInvert->setChecked (config->readBoolEntry("Invertation", false));
+  slotToggleInvert();
+
   unsigned int rotationIndex = config->readUnsignedNumEntry("RotationIndex", 0);
   setRotationIndex(rotationIndex);
   emit updateRotationIndex(m_rotationIndex);
@@ -322,14 +357,6 @@ void KmagApp::readOptions()
   setFPSIndex(fpsIndex);
   emit updateFPSIndex(m_fpsIndex);
 
-  QString mode = config->readEntry("Mode", "followmouse");
-  if (mode == "wholescreen")
-    slotModeWholeScreen();
-  else if (mode == "selectionwindow")
-    slotModeSelWin();
-  else
-    slotModeFollowMouse();
-
   QRect defaultRect(0,0,211,164);
   m_zoomView->setSelRectPos(config->readRectEntry("SelRect", &defaultRect));
 
@@ -360,6 +387,23 @@ void KmagApp::readOptions()
 
   m_pShowSettingsToolBar->setChecked(config->readBoolEntry("ShowSettingsToolBar", true));
   slotShowSettingsToolBar();
+
+  config->setGroup("General Options");
+
+  edgesize = config->readNumEntry("EdgeSize", 0);
+  QString mode = config->readEntry("Mode", "followmouse");
+  if (mode == "selectionwindow")
+    slotModeSelWin();
+  else if (mode == "edgetop")
+    setEdgeMode (m_modeEdgeTop);
+  else if (mode == "edgeleft")
+    setEdgeMode (m_modeEdgeLeft);
+  else if (mode == "edgeright")
+    setEdgeMode (m_modeEdgeRight);
+  else if (mode == "edgebottom")
+    setEdgeMode (m_modeEdgeBottom);
+  else
+    slotModeFollowMouse();
 }
 
 bool KmagApp::queryClose()
@@ -373,21 +417,6 @@ bool KmagApp::queryExit()
   return true;
 }
 
-/**
- * Called when mouse is clicked inside the window
- *
- * @param e
- */
-void KmagApp::contextMenuEvent ( QContextMenuEvent * e )
-{
- // show popup
- KXMLGUIFactory *factory = this->factory();
- QPopupMenu *popup = (QPopupMenu *)factory->container("mainPopUp",this);
- if (popup != 0)
-   popup->popup(e->globalPos(), 0);
- e->accept();
-}
-
 
 /////////////////////////////////////////////////////////////////////
 // SLOT IMPLEMENTATION
@@ -464,6 +493,14 @@ void KmagApp::setZoomIndex(int index)
   emit updateZoomValue(zoomArray[m_zoomIndex]);
 }
 
+/**
+ * Shows/hides the mouse cursor
+ */
+void KmagApp::slotToggleInvert ()
+{
+  m_zoomView->setInvertation (m_pInvert->isChecked());
+}
+
 /**
  * Sets the rotation index to index
  */
@@ -572,37 +609,172 @@ void KmagApp::slotToggleRefresh()
 }
 
 
-void KmagApp::slotModeWholeScreen()
-{
-  m_zoomView->setSelRectPos(QRect (0, 0, QApplication::desktop()->width(), QApplication::desktop()->height()));
-  m_zoomView->followMouse(false);
-  m_zoomView->showSelRect(false);
-  m_zoomView->setFitToWindow (false);
-  m_modeFollowMouse->setChecked(false);
-  m_modeWholeScreen->setChecked(true);
-  m_modeSelWin->setChecked(false);
-}
-
-
 void KmagApp::slotModeSelWin()
 {
+  unsetEdgeMode(m_modeSelWin);
+
+  m_zoomView->fitToWindow();
+  m_zoomView->setFitToWindow (false);
   m_zoomView->followMouse(false);
   m_zoomView->showSelRect(true);
-  m_zoomView->setFitToWindow (false);
-  m_modeFollowMouse->setChecked(false);
-  m_modeWholeScreen->setChecked(false);
-  m_modeSelWin->setChecked(true);
 }
 
 
 void KmagApp::slotModeFollowMouse()
 {
+  unsetEdgeMode(m_modeFollowMouse);
+
+  m_zoomView->setFitToWindow (true);
   m_zoomView->followMouse(true);
   m_zoomView->showSelRect(false);
-  m_zoomView->setFitToWindow (true);
-  m_modeFollowMouse->setChecked(true);
-  m_modeWholeScreen->setChecked(false);
+}
+
+
+void KmagApp::slotModeEdgeTop()
+{
+  // ask for edgesize
+  int newedgesize; bool ok;
+  newedgesize = KInputDialog::getInteger (i18n ("Magnify to Screen Edge - Select Size"),
+                                          i18n ("Size:"), edgesize > 0 ? edgesize : 300, 200,
+                                          QApplication::desktop()->screenGeometry().height()/2,
+                                          25, 10, &ok, 0, "getedgesize");
+
+  if (ok) {
+    edgesize = newedgesize;
+    setEdgeMode(m_modeEdgeTop);
+  }
+}
+
+
+void KmagApp::slotModeEdgeLeft()
+{
+  // ask for edgesize
+  int newedgesize; bool ok;
+  newedgesize = KInputDialog::getInteger (i18n ("Magnify to Left Screen Edge - Select Size"),
+                                          i18n ("Size:"), edgesize > 0 ? edgesize : 300, 200,
+                                          QApplication::desktop()->screenGeometry().width()/2,
+                                          25, 10, &ok, 0, "getedgesize");
+
+  if (ok) {
+    edgesize = newedgesize;
+    setEdgeMode(m_modeEdgeLeft);
+  }
+}
+
+
+void KmagApp::slotModeEdgeRight()
+{
+  // ask for edgesize
+  int newedgesize; bool ok;
+  newedgesize = KInputDialog::getInteger (i18n ("Magnify to Right Screen Edge - Select Size"),
+                                          i18n ("Size:"), edgesize > 0 ? edgesize : 300, 200,
+                                          QApplication::desktop()->screenGeometry().width()/2,
+                                          25, 10, &ok, 0, "getedgesize");
+
+  if (ok) {
+    edgesize = newedgesize;
+    setEdgeMode(m_modeEdgeRight);
+  }
+}
+
+
+void KmagApp::slotModeEdgeBottom()
+{
+  // ask for edgesize
+  int newedgesize; bool ok;
+  newedgesize = KInputDialog::getInteger (i18n ("Magnify to Bottom Screen Edge - Select Size"),
+                                          i18n ("Size:"), edgesize > 0 ? edgesize : 300, 200,
+                                          QApplication::desktop()->screenGeometry().height()/2,
+                                          25, 10, &ok, 0, "getedgesize");
+
+  if (ok) {
+    edgesize = newedgesize;
+    setEdgeMode(m_modeEdgeBottom);
+  }
+}
+
+
+void KmagApp::setEdgeMode (KToggleAction *mode)
+{
+  if (m_modeEdgeLeft || mode == m_modeEdgeRight) {
+    if (edgesize < 200 || edgesize > QApplication::desktop()->screenGeometry().width()/2)
+      edgesize = QApplication::desktop()->screenGeometry().width()/4;
+  } else {
+    if (edgesize < 200 || edgesize > QApplication::desktop()->screenGeometry().height()/2)
+      edgesize = QApplication::desktop()->height()/4;
+  }
+
+  m_modeFollowMouse->setChecked(false);
   m_modeSelWin->setChecked(false);
+  m_modeEdgeTop->setChecked(false);
+  m_modeEdgeLeft->setChecked(false);
+  m_modeEdgeRight->setChecked(false);
+  m_modeEdgeBottom->setChecked(false);
+  mode->setChecked (true);
+
+  m_zoomView->setFitToWindow (true);
+  m_zoomView->followMouse(true);
+  m_zoomView->showSelRect(false);
+
+  m_zoomView->reparent (0, QPoint(), true);
+  KWin::setType(m_zoomView->winId(), NET::Dock);
+  KWin::setState(m_zoomView->winId(), NET::Sticky | NET::KeepBelow | NET::SkipTaskbar | NET::SkipPager);
+  KWin::setOnAllDesktops(m_zoomView->winId(), true);
+
+  m_pShowMenu->setEnabled (false);
+  m_pShowMainToolBar->setEnabled (false);
+  m_pShowViewToolBar->setEnabled (false);
+  m_pShowSettingsToolBar->setEnabled (false);
+
+  hide();
+
+  if (mode == m_modeEdgeTop) {
+    m_zoomView->setGeometry (0, 0, QApplication::desktop()->screenGeometry().width(), edgesize);
+    KWin::setExtendedStrut (m_zoomView->winId(), 0, 0, 0, 0, 0, 0,
+                            edgesize, 0, QApplication::desktop()->screenGeometry().width(),
+                            0, 0, 0);
+  } else if (mode == m_modeEdgeLeft) {
+    m_zoomView->setGeometry (0, 0, edgesize, QApplication::desktop()->screenGeometry().height());
+    KWin::setExtendedStrut (m_zoomView->winId(),
+                            edgesize, 0, QApplication::desktop()->screenGeometry().height(),
+                            0, 0, 0, 0, 0, 0, 0, 0, 0);
+  } else if (mode == m_modeEdgeRight) {
+    m_zoomView->setGeometry (QApplication::desktop()->screenGeometry().width()-edgesize, 0,
+                 edgesize, QApplication::desktop()->screenGeometry().height());
+    KWin::setExtendedStrut (m_zoomView->winId(), 0, 0, 0,
+                            edgesize, 0, QApplication::desktop()->screenGeometry().height(),
+                            0, 0, 0, 0, 0, 0);
+  } else {
+    m_zoomView->setGeometry (0, QApplication::desktop()->screenGeometry().height()-edgesize,
+                 QApplication::desktop()->screenGeometry().width(), edgesize);
+    KWin::setExtendedStrut (m_zoomView->winId(), 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                            edgesize, 0, QApplication::desktop()->screenGeometry().width());
+  }
+}
+
+
+void KmagApp::unsetEdgeMode (KToggleAction *mode)
+{
+  edgesize = 0;
+
+  m_modeFollowMouse->setChecked(false);
+  m_modeSelWin->setChecked(false);
+  m_modeEdgeTop->setChecked(false);
+  m_modeEdgeLeft->setChecked(false);
+  m_modeEdgeRight->setChecked(false);
+  m_modeEdgeBottom->setChecked(false);
+  mode->setChecked (true);
+
+  m_zoomView->reparent (this, QPoint(), true);
+  setCentralWidget(m_zoomView);
+  KWin::setExtendedStrut (winId(), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
+
+  m_pShowMenu->setEnabled (true);
+  m_pShowMainToolBar->setEnabled (true);
+  m_pShowViewToolBar->setEnabled (true);
+  m_pShowSettingsToolBar->setEnabled (true);
+
+  show();
 }
 
 
@@ -738,7 +910,7 @@ void KmagApp::slotShowSettingsToolBar()
 {
   ///////////////////////////////////////////////////////////////////
   // turn viewToolbar on or off
-  if(!m_pShowSettingsToolBar->isChecked())
+  if(!m_pShowSettingsToolBar->isChecked() || edgesize > 0)
   {
     toolBar("settingsToolBar")->hide();
   }
@@ -763,6 +935,16 @@ void KmagApp::slotEditToolbars()
 }
 
 
+void KmagApp::contextMenu (QPoint pos)
+{
+ // show popup
+ KXMLGUIFactory *factory = this->factory();
+ QPopupMenu *popup = (QPopupMenu *)factory->container("mainPopUp",this);
+ if (popup != 0)
+   popup->popup(pos, 0);
+}
+
+
 void KmagApp::slotNewToolbarConfig()
 {
   applyMainWindowSettings( KGlobal::config(), "MainWindow" );
