@@ -5,14 +5,20 @@
   (C) 2005 by Gary Cramblitt <garycramblitt@comcast.net>
   -------------------
   Original author: Gary Cramblitt <garycramblitt@comcast.net>
- ******************************************************************************/
 
-/******************************************************************************
- *                                                                            *
- *    This program is free software; you can redistribute it and/or modify    *
- *    it under the terms of the GNU General Public License as published by    *
- *    the Free Software Foundation; version 2 of the License.                 *
- *                                                                            *
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
+
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  ******************************************************************************/
 
 // Qt includes.
@@ -207,8 +213,8 @@ QString StringReplacerConf::loadFromFile( const QString& filename, bool clear)
     QListViewItem* item = 0;
     if ( !clear ) item = m_widget->substLView->lastChild();
     QDomNodeList wordList = doc.elementsByTagName("word");
-    int wordListCount = wordList.count();
-    for (int wordIndex = 0; wordIndex < wordListCount; wordIndex++)
+    const int wordListCount = wordList.count();
+    for (int wordIndex = 0; wordIndex < wordListCount; ++wordIndex)
     {
         // kdDebug() << "StringReplacerConf::load: start parsing of word " << wordIndex << endl;
         QDomNode wordNode = wordList.item(wordIndex);
@@ -216,8 +222,8 @@ QString StringReplacerConf::loadFromFile( const QString& filename, bool clear)
         QString wordType;
         QString match;
         QString subst;
-        int propListCount = propList.count();
-        for (int propIndex = 0; propIndex < propListCount; propIndex++)
+        const int propListCount = propList.count();
+        for (int propIndex = 0; propIndex < propListCount; ++propIndex)
         {
             QDomNode propNode = propList.item(propIndex);
             QDomElement prop = propNode.toElement();
@@ -426,8 +432,8 @@ void StringReplacerConf::slotLanguageBrowseButton_clicked()
     // Blank line so user can select no language.
     QListViewItem* item = new KListViewItem(langLView, "", "");
     if (m_languageCodeList.isEmpty()) item->setSelected(true);
-    int allLocalesCount = allLocales.count();
-    for (int ndx=0; ndx < allLocalesCount; ndx++)
+    const int allLocalesCount = allLocales.count();
+    for (int ndx=0; ndx < allLocalesCount; ++ndx)
     {
         locale = allLocales[ndx];
         KGlobal::locale()->splitLocale(locale, languageCode, countryCode, charSet);
