@@ -2,7 +2,7 @@
   A KPart to display running jobs in KTTSD and permit user to stop, rewind,
   advance, change Talker, etc. 
   -------------------
-  Copyright : (C) 2004 by Gary Cramblitt <garycramblitt@comcast.net>
+  Copyright : (C) 2004,2005 by Gary Cramblitt <garycramblitt@comcast.net>
   -------------------
   Current Maintainer: Gary Cramblitt <garycramblitt@comcast.net>
  ******************************************************************************/
@@ -16,45 +16,37 @@
  ***************************************************************************/
 
 // QT includes.
-#include <qdockarea.h>
 #include <qvbox.h>
-#include <qgrid.h>
+#include <qhbox.h>
 #include <qlabel.h>
 #include <qsplitter.h>
 #include <qclipboard.h>
+#include <qpushbutton.h>
+#include <qobjectlist.h>
+#include <qwhatsthis.h>
+
+#include <qmime.h>
 
 // KDE includes.
 #include <kinstance.h>
 #include <klocale.h>
 #include <kaboutdata.h>
 #include <klistview.h>
-#include <kaction.h>
-#include <ktoolbar.h>
 #include <kiconloader.h>
 #include <kdebug.h>
 #include <kencodingfiledialog.h>
 #include <kapplication.h>
 #include <kinputdialog.h>
+#include <ktextedit.h>
 
 // KTTS includes.
 #include "kspeech.h"
 #include "talkercode.h"
-#include "selecttalkerwidget.h"
+#include "selecttalkerdlg.h"
 #include "kttsjobmgr.h"
 #include "kttsjobmgr.moc"
 
-extern "C"
-{
-    /**
-    * This function is the 'main' function of this part.  It takes
-    * the form 'void *init_lib<library name>()  It always returns a
-    * new factory object
-    */
-    KDE_EXPORT void *init_libkttsjobmgrpart()
-    {
-        return new KttsJobMgrFactory;
-    }
-};
+K_EXPORT_COMPONENT_FACTORY( libkttsjobmgrpart, KttsJobMgrFactory )
 
 /**
 * We need one static instance of the factory for our C 'main'
@@ -100,41 +92,17 @@ KttsJobMgrPart::KttsJobMgrPart(QWidget *parent, const char *name) :
     KParts::ReadOnlyPart(parent, name)
 {
     // Initialize some variables.
-    selectOnTextSet = false;
-    
+    m_selectOnTextSet = false;
+    m_buttonBox = 0;
+
     setInstance(KttsJobMgrFactory::instance());
-    
+
+    // All the ktts components use the same catalogue.
+    KGlobal::locale()->insertCatalogue("kttsd");
+
     // Create a QVBox to host everything.
     QVBox* vBox = new QVBox(parent);
-
-    // Create a QDockArea to host the toolbar.
-    QDockArea* toolBarDockArea = new QDockArea(Qt::Horizontal, QDockArea::Normal, vBox, "jobmgr_toolbar_dockarea");
-
-    // Create three KToolBars.
-    m_toolBar1 = new KToolBar(vBox, "jobmgr_toolbar1");
-    m_toolBar1->setIconText(KToolBar::IconTextRight);
-    m_toolBar1->setTitle(i18n("Text-to-Speech Job Manager Toolbar"));
-    m_toolBar1->setMovingEnabled(true);
-    // This is a temporary workaround until these toolbars size correctly when floated.
-    m_toolBar1->setResizeEnabled(true);
-    m_toolBar2 = new KToolBar(vBox, "jobmgr_toolbar2");
-    m_toolBar2->setIconText(KToolBar::IconTextRight);
-    m_toolBar2->setTitle(i18n("Text-to-Speech Job Manager Toolbar"));
-    m_toolBar2->setMovingEnabled(true);
-    m_toolBar2->setResizeEnabled(true);
-    m_toolBar3 = new KToolBar(vBox, "jobmgr_toolbar3");
-    m_toolBar3->setIconText(KToolBar::IconTextRight);
-    m_toolBar3->setTitle(i18n("Text-to-Speech Job Manager Toolbar"));
-    m_toolBar3->setMovingEnabled(true);
-    m_toolBar3->setResizeEnabled(true);
-
-    // Add the toolbars to the QDockArea.
-    toolBarDockArea->setAcceptDockWindow(m_toolBar1, true);
-    toolBarDockArea->moveDockWindow(m_toolBar1);
-    toolBarDockArea->setAcceptDockWindow(m_toolBar2, true);
-    toolBarDockArea->moveDockWindow(m_toolBar2);
-    toolBarDockArea->setAcceptDockWindow(m_toolBar3, true);
-    toolBarDockArea->moveDockWindow(m_toolBar3);
+    vBox->setMargin(6);
 
     // Create a splitter to contain the Job List View and the current sentence.
     QSplitter* splitter = new QSplitter(vBox);
@@ -155,11 +123,154 @@ KttsJobMgrPart::KttsJobMgrPart(QWidget *parent, const char *name) :
     // Do not sort the list.
     m_jobListView->setSorting(-1);
 
-    // Splitter to resize Job ListView to minimum height.
-    splitter->setResizeMode(m_jobListView, QSplitter::Stretch);
+    QString jobListViewWT = i18n(
+            "<p>These are all the text jobs.  The <b>State</b> column "
+            "may be:"
+            "<ul>"
+            "<li><b>Queued</b> - the job is waiting and will not be spoken until its state "
+            "is changed to <b>Waiting</b> by clicking the <b>Resume</b> or <b>Restart</b> buttons.</li>"
+            "<li><b>Waiting</b> - the job is ready to be spoken.  It will be spoken when the jobs "
+            "preceding it in the list have finished.</li>"
+            "<li><b>Speaking</b> - the job is speaking.  The <b>Position</b> column shows the current "
+            "sentence of the job being spoken.  You may pause a speaking job by clicking the "
+            "<b>Hold</b> button.</li>"
+            "<li><b>Paused</b> - the job is currently paused.  Paused jobs prevent jobs below them "
+            "from speaking.  Use the <b>Resume</b> or <b>Restart</b> buttons to resume speaking the "
+            "job, or click <b>Later</b> to move the job down in the list.</li>"
+            "<li><b>Finished</b> - the job has finished speaking.  When a second job finishes, "
+            "this one will be deleted.  You may click <b>Restart</b> to repeat the job.</li>"
+            "</ul>"
+            "<em>Note</em>: Messages, Warnings, and Screen Reader Output do not appear in this list.  "
+            "See the Handbook for more information."
+            "</p>");
+    QWhatsThis::add(m_jobListView, jobListViewWT);
+
+    // splitter->setResizeMode(m_jobListView, QSplitter::Stretch);
+
+    // Create a VBox to hold buttons and current sentence.
+    QVBox* bottomBox = new QVBox(splitter);
+
+    // Create a box to hold buttons.
+    m_buttonBox = new QVBox(bottomBox);
+    m_buttonBox->setSpacing(6);
+
+    // Create 3 HBoxes to host buttons.
+    QHBox* hbox1 = new QHBox(m_buttonBox);
+    hbox1->setSpacing(6);
+    QHBox* hbox2 = new QHBox(m_buttonBox);
+    hbox2->setSpacing(6);
+    QHBox* hbox3 = new QHBox(m_buttonBox);
+    hbox3->setSpacing(6);
+
+    // Do not let button box stretch vertically.
+    m_buttonBox->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
+
+    // All the buttons with "job_" at start of their names will be enabled/disabled when a job is
+    // selected in the Job List View.
+    // All the buttons with "part_" at the start of their names will be enabled/disabled when a
+    // job is selected in the Job List View that has multiple parts.
+
+    QPushButton* btn;
+    QString wt;
+    btn = new QPushButton(KGlobal::iconLoader()->loadIconSet("stop", KIcon::Small, 0, true),
+                          i18n("Hold"), hbox1, "job_hold");
+    wt = i18n(
+            "<p>Changes a job to Paused state.  If currently speaking, the job stops speaking.  "
+            "Paused jobs prevent jobs that follow them from speaking, so either click "
+            "<b>Resume</b> to make the job speakable, or click <b>Later</b> to move it "
+            "down in the list.</p>");
+    QWhatsThis::add(btn, wt);
+    connect (btn, SIGNAL(clicked()), this, SLOT(slot_job_hold()));
+    btn = new QPushButton(KGlobal::iconLoader()->loadIconSet("exec", KIcon::Small, 0, true),
+                          i18n("Resume"), hbox1, "job_resume");
+    wt = i18n(
+            "<p>Resumes a paused job or changes a Queued job to Waiting.  If the job is the "
+            "top speakable job in the list, it begins speaking.</p>");
+    QWhatsThis::add(btn, wt);
+    connect (btn, SIGNAL(clicked()), this, SLOT(slot_job_resume()));
+    btn = new QPushButton(KGlobal::iconLoader()->loadIconSet("redo", KIcon::Small, 0, true),
+                          i18n("R&estart"), hbox1, "job_restart");
+    wt = i18n(
+            "<p>Rewinds a job to the beginning and changes its state to Waiting.  If the job "
+            "is the top speakable job in the list, it begins speaking.</p>");
+    QWhatsThis::add(btn, wt);
+    connect (btn, SIGNAL(clicked()), this, SLOT(slot_job_restart()));
+    btn = new QPushButton(KGlobal::iconLoader()->loadIconSet("edittrash", KIcon::Small, 0, true),
+                          i18n("Re&move"), hbox1, "job_remove");
+    wt = i18n(
+            "<p>Deletes the job.  If it is currently speaking, it stops speaking.  The next "
+            "speakable job in the list begins speaking.</p>");
+    QWhatsThis::add(btn, wt);
+    connect (btn, SIGNAL(clicked()), this, SLOT(slot_job_remove()));
+    btn = new QPushButton(KGlobal::iconLoader()->loadIconSet("down", KIcon::Small, 0, true),
+                          i18n("&Later"), hbox1, "job_later");
+    wt = i18n(
+            "<p>Moves a job downward in the list so that it will be spoken later.  If the job "
+            "is currently speaking, its state changes to Paused.</p>");
+    QWhatsThis::add(btn, wt);
+    connect (btn, SIGNAL(clicked()), this, SLOT(slot_job_move()));
+
+    btn = new QPushButton(KGlobal::iconLoader()->loadIconSet("2leftarrow", KIcon::Small, 0, true),
+                          i18n("Pre&vious Part"), hbox2, "part_prevpart");
+    wt = i18n(
+            "<p>Rewinds a multi-part job to the previous part.</p>");
+    QWhatsThis::add(btn, wt);
+    connect (btn, SIGNAL(clicked()), this, SLOT(slot_job_prev_par()));
+    btn = new QPushButton(KGlobal::iconLoader()->loadIconSet("1leftarrow", KIcon::Small, 0, true),
+                          i18n("&Previous Sentence"), hbox2, "job_prevsentence");
+    wt = i18n(
+            "<p>Rewinds a job to the previous sentence.</p>");
+    QWhatsThis::add(btn, wt);
+    connect (btn, SIGNAL(clicked()), this, SLOT(slot_job_prev_sen()));
+    btn = new QPushButton(KGlobal::iconLoader()->loadIconSet("1rightarrow", KIcon::Small, 0, true),
+                          i18n("&Next Sentence"), hbox2, "job_nextsentence");
+    wt = i18n(
+            "<p>Advances a job to the next sentence.</p>");
+    QWhatsThis::add(btn, wt);
+    connect (btn, SIGNAL(clicked()), this, SLOT(slot_job_next_sen()));
+    btn = new QPushButton(KGlobal::iconLoader()->loadIconSet("2rightarrow", KIcon::Small, 0, true),
+                          i18n("Ne&xt Part"), hbox2, "part_nextpart");
+    wt = i18n(
+            "<p>Advances a multi-part job to the next part.</p>");
+    QWhatsThis::add(btn, wt);
+    connect (btn, SIGNAL(clicked()), this, SLOT(slot_job_next_par()));
+
+    btn = new QPushButton(KGlobal::iconLoader()->loadIconSet("klipper", KIcon::Small, 0, true),
+                          i18n("&Speak Clipboard"), hbox3, "speak_clipboard");
+    wt = i18n(
+            "<p>Queues the current contents of the clipboard for speaking and sets its state "
+            "to Waiting.  If the job is the topmost in the list, it begins speaking.  "
+            "The job will be spoken by the topmost Talker in the <b>Talkers</b> tab.</p>");
+    QWhatsThis::add(btn, wt);
+    connect (btn, SIGNAL(clicked()), this, SLOT(slot_speak_clipboard()));
+    btn = new QPushButton(KGlobal::iconLoader()->loadIconSet("fileopen", KIcon::Small, 0, true),
+                          i18n("Spea&k File"), hbox3, "speak_file");
+    wt = i18n(
+            "<p>Prompts you for a file name and queues the contents of the file for speaking.  "
+            "You must click the <b>Resume</b> button before the job will be speakable.  "
+            "The job will be spoken by the topmost Talker in the <b>Talkers</b> tab.</p>");
+    QWhatsThis::add(btn, wt);
+    connect (btn, SIGNAL(clicked()), this, SLOT(slot_speak_file()));
+    btn = new QPushButton(KGlobal::iconLoader()->loadIconSet("translate", KIcon::Small, 0, true),
+                          i18n("Change Talker"), hbox3, "job_changetalker");
+    wt = i18n(
+            "<p>Prompts you with a list of your configured Talkers from the <b>Talkers</b> tab.  "
+            "The job will be spoken using the selected Talker.</p>");
+    QWhatsThis::add(btn, wt);
+    connect (btn, SIGNAL(clicked()), this, SLOT(slot_job_change_talker()));
+    btn = new QPushButton(KGlobal::iconLoader()->loadIconSet("reload_page", KIcon::Small, 0, true),
+                          i18n("&Refresh"), hbox3, "refresh");
+    wt = i18n(
+            "<p>Refresh the list of jobs.</p>");
+    QWhatsThis::add(btn, wt);
+    connect (btn, SIGNAL(clicked()), this, SLOT(slot_refresh()));
+
+    // Disable job buttons until a job is selected.
+    enableJobActions(false);
+    enableJobPartActions(false);
 
     // Create a VBox for the current sentence and sentence label.
-    QVBox* sentenceVBox = new QVBox(splitter);
+    QVBox* sentenceVBox = new QVBox(bottomBox);
 
     // Create a label for current sentence.
     QLabel* currentSentenceLabel = new QLabel(sentenceVBox);
@@ -167,16 +278,19 @@ KttsJobMgrPart::KttsJobMgrPart(QWidget *parent, const char *name) :
     currentSentenceLabel->setText(i18n("Current Sentence"));
 
     // Create a box to contain the current sentence.
-    m_currentSentence = new QLabel(sentenceVBox);
-    m_currentSentence->setFrameStyle(QFrame::Panel | QFrame::Sunken);
-    m_currentSentence->setAlignment(Qt::AlignAuto | Qt::AlignVCenter | Qt::WordBreak);
+    m_currentSentence = new KTextEdit(sentenceVBox);
+    m_currentSentence->setReadOnly(true);
+    m_currentSentence->setWordWrap(QTextEdit::WidgetWidth);
+    m_currentSentence->setWrapPolicy(QTextEdit::AtWordOrDocumentBoundary);
+    m_currentSentence->setHScrollBarMode(QScrollView::AlwaysOff);
+    m_currentSentence->setVScrollBarMode(QScrollView::Auto);
+    wt = i18n(
+            "<p>The text of the sentence currently speaking.</p>");
+    QWhatsThis::add(m_currentSentence, wt);
 
     // Set the main widget for the part.
     setWidget(vBox);
 
-    // Set up toolbar buttons.
-    setupActions();
-
     connect(m_jobListView, SIGNAL(selectionChanged(QListViewItem* )),
         this, SLOT(slot_selectionChanged(QListViewItem* )));
 
@@ -234,10 +348,18 @@ KttsJobMgrPart::KttsJobMgrPart(QWidget *parent, const char *name) :
     m_extension = new KttsJobMgrBrowserExtension(this);
 
     m_jobListView->show();
+
+    // Divide splitter in half.  ListView gets half.  Buttons and Current Sentence get half.
+    int halfSplitterSize = splitter->height()/2;
+    QValueList<int> splitterSizes;
+    splitterSizes.append(halfSplitterSize);
+    splitterSizes.append(halfSplitterSize);
+    splitter->setSizes(splitterSizes);
 }
 
 KttsJobMgrPart::~KttsJobMgrPart()
 {
+    KGlobal::locale()->removeCatalogue("kttsd");
     closeURL();
 }
 
@@ -251,94 +373,18 @@ bool KttsJobMgrPart::closeURL()
     return true;
 }
 
-/**
-* Set up toolbar.
-*/
-void KttsJobMgrPart::setupActions()
-{
-//    setXMLFile("kttsjobmgrui.rc");
-
-    KAction* act;
-
-    // All the buttons with "job_" at start of their names will be enabled/disabled when a job is
-    // selected in the Job List View.
-    // All the buttons with "part_" at the start of their names will be enabled/disabled when a
-    // job is selected in the Job List View that has multiple parts.
-
-    act = new KAction(i18n("&Hold"),
-        KGlobal::iconLoader()->loadIconSet("stop", KIcon::Toolbar, 0, true),
-        0, this, SLOT(slot_job_hold()), actionCollection(), "job_hold");
-    act->plug(m_toolBar1);
-    act = new KAction(i18n("&Resume"),
-        KGlobal::iconLoader()->loadIconSet("exec", KIcon::Toolbar, 0, true),
-        0, this, SLOT(slot_job_resume()), actionCollection(), "job_resume");
-    act->plug(m_toolBar1);
-    act = new KAction(i18n("R&estart"),
-        KGlobal::iconLoader()->loadIconSet("redo", KIcon::Toolbar, 0, true),
-        0, this, SLOT(slot_job_restart()), actionCollection(), "job_restart");
-    act->plug(m_toolBar1);
-    act = new KAction(i18n("Remove"),
-        KGlobal::iconLoader()->loadIconSet("edittrash", KIcon::Toolbar, 0, true),
-        0, this, SLOT(slot_job_remove()), actionCollection(), "job_remove");
-    act->plug(m_toolBar1);
-    act = new KAction(i18n("Later"),
-        KGlobal::iconLoader()->loadIconSet("down", KIcon::Toolbar, 0, true),
-        0, this, SLOT(slot_job_move()), actionCollection(), "job_move");
-    act->plug(m_toolBar1);
-    act = new KAction(i18n("Change Talker"),
-        KGlobal::iconLoader()->loadIconSet("translate", KIcon::Toolbar, 0, true),
-        0, this, SLOT(slot_job_change_talker()), actionCollection(), "job_change_talker");
-    act->plug(m_toolBar1);
-
-    act = new KAction(i18n("Previous Part"),
-        KGlobal::iconLoader()->loadIconSet("2leftarrow", KIcon::Toolbar, 0, true),
-        0, this, SLOT(slot_job_prev_par()), actionCollection(), "part_prev_par");
-    act->plug(m_toolBar2);
-    act = new KAction(i18n("Previous Sentence"),
-        KGlobal::iconLoader()->loadIconSet("1leftarrow", KIcon::Toolbar, 0, true),
-        0, this, SLOT(slot_job_prev_sen()), actionCollection(), "job_prev_sen");
-    act->plug(m_toolBar2);
-    act = new KAction(i18n("Next Sentence"),
-        KGlobal::iconLoader()->loadIconSet("1rightarrow", KIcon::Toolbar, 0, true),
-        0, this, SLOT(slot_job_next_sen()), actionCollection(), "job_next_sen");
-    act->plug(m_toolBar2);
-    act = new KAction(i18n("Next Part"),
-        KGlobal::iconLoader()->loadIconSet("2rightarrow", KIcon::Toolbar, 0, true),
-        0, this, SLOT(slot_job_next_par()), actionCollection(), "part_next_par");
-    act->plug(m_toolBar2);
-
-    act = new KAction(i18n("Speak Clipboard"),
-        KGlobal::iconLoader()->loadIconSet("klipper", KIcon::Toolbar, 0, true),
-        0, this, SLOT(slot_speak_clipboard()), actionCollection(), "speak_clipboard");
-    act->plug(m_toolBar3);
-    act = new KAction(i18n("Speak File"),
-        KGlobal::iconLoader()->loadIconSet("fileopen", KIcon::Toolbar, 0, true),
-        0, this, SLOT(slot_speak_file()), actionCollection(), "speak_file");
-    act->plug(m_toolBar3);
-    act = new KAction(i18n("Refresh"),
-        KGlobal::iconLoader()->loadIconSet("reload_page", KIcon::Toolbar, 0, true),
-        0, this, SLOT(slot_refresh()), actionCollection(), "refresh");
-    act->plug(m_toolBar3);
-
-    // Disable job buttons until a job is selected.
-//    stateChanged("no_job_selected");
-    enableJobActions(false);
-    enableJobPartActions(false);
-}
-
 /**
 * This slot is connected to the Job List View selectionChanged signal.
 */
 void KttsJobMgrPart::slot_selectionChanged(QListViewItem*)
 {
     // Enable job buttons.
-//    stateChanged("job_selected");
     enableJobActions(true);
     enableJobPartActions((getCurrentJobPartCount() > 1));
 }
 
 /**
-* Slots connected to toolbar buttons.
+* Slots connected to buttons.
 */
 void KttsJobMgrPart::slot_job_hold()
 {
@@ -363,7 +409,7 @@ void KttsJobMgrPart::slot_job_resume()
 void KttsJobMgrPart::slot_job_restart()
 {
     uint jobNum = getCurrentJobNum();
-    kdDebug() << "KttsJobMgrPart::slot_job_restart: jobNum = " << jobNum << endl;
+    // kdDebug() << "KttsJobMgrPart::slot_job_restart: jobNum = " << jobNum << endl;
     if (jobNum)
     {
         startText(jobNum);
@@ -421,6 +467,7 @@ void KttsJobMgrPart::slot_job_remove()
     if (jobNum)
     {
         removeText(jobNum);
+        m_currentSentence->clear();
     }
 }
 
@@ -437,168 +484,23 @@ void KttsJobMgrPart::slot_job_move()
     }
 }
 
-QString KttsJobMgrPart::translatedGender(const QString &gender)
-{
-    if (gender == "male")
-        return i18n("male");
-    else if (gender == "female")
-        return i18n("female");
-    else if (gender == "neutral")
-        return i18n("neutral gender", "neutral");
-    else return gender;
-}
-
-QString KttsJobMgrPart::translatedVolume(const QString &volume)
-{
-    if (volume == "medium")
-        return i18n("medium sound", "medium");
-    else if (volume == "loud")
-        return i18n("loud sound", "loud");
-    else if (volume == "soft")
-        return i18n("soft sound", "soft");
-    else return volume;
-}
-
-QString KttsJobMgrPart::translatedRate(const QString &rate)
-{
-    if (rate == "medium")
-        return i18n("medium speed", "medium");
-    else if (rate == "fast")
-        return i18n("fast speed", "fast");
-    else if (rate == "slow")
-        return i18n("slow speed", "slow");
-    else return rate;
-}
-
-/**
- * Given a talker code, parses out the attributes.
- * @param talkerCode       The talker code.
- * @return languageCode    Language Code.
- * @return voice           Voice name.
- * @return gender          Gender.
- * @return volume          Volume.
- * @return rate            Rate.
- * @return plugInName      Name of Synthesizer plugin.
- */
-void KttsJobMgrPart::parseTalkerCode(const QString &talkerCode,
-    QString &languageCode,
-    QString &voice,
-    QString &gender,
-    QString &volume,
-    QString &rate,
-    QString &plugInName)
-{
-    languageCode = talkerCode.section("lang=", 1, 1);
-    languageCode = languageCode.section('"', 1, 1);
-    voice = talkerCode.section("name=", 1, 1);
-    voice = voice.section('"', 1, 1);
-    gender = talkerCode.section("gender=", 1, 1);
-    gender = gender.section('"', 1, 1);
-    volume = talkerCode.section("volume=", 1, 1);
-    volume = volume.section('"', 1, 1);
-    rate = talkerCode.section("rate=", 1, 1);
-    rate = rate.section('"', 1, 1);
-    plugInName = talkerCode.section("synthesizer=", 1, 1);
-    plugInName = plugInName.section('"', 1, 1);
-}
-
-/**
- * Converts a language code plus optional country code to language description.
- */
-QString KttsJobMgrPart::languageCodeToLanguage(const QString &languageCode)
-{
-    QString twoAlpha;
-    QString countryCode;
-    QString charSet;
-    QString language;
-    if (languageCode == "other")
-        language = i18n("Other");
-    else
-    {
-        KGlobal::locale()->splitLocale(languageCode, twoAlpha, countryCode, charSet);
-        language = KGlobal::locale()->twoAlphaToLanguageName(twoAlpha);
-    }
-    if (!countryCode.isEmpty())
-        language += " (" + KGlobal::locale()->twoAlphaToCountryName(countryCode) + ")";
-    return language;
-}
-
-/**
-* Convert a Talker Code to a translated, displayable name.
-*/
-QString KttsJobMgrPart::talkerCodeToDisplayName(const QString &talkerCode)
-{
-    QString languageCode;
-    QString voice;
-    QString gender;
-    QString volume;
-    QString rate;
-    QString plugInName;
-    parseTalkerCode(talkerCode, languageCode, voice, gender, volume, rate, plugInName);
-    QString display;
-    if (!languageCode.isEmpty()) display = languageCodeToLanguage(languageCode);
-    if (!plugInName.isEmpty()) display += "," + plugInName;
-    if (!voice.isEmpty()) display += "," + voice;
-    if (!gender.isEmpty()) display += "," + translatedGender(gender);
-    if (!volume.isEmpty()) display += "," + translatedVolume(volume);
-    if (!rate.isEmpty()) display += "," + translatedRate(rate);
-    return display;
-}
-
 void KttsJobMgrPart::slot_job_change_talker()
 {
-    uint jobNum = getCurrentJobNum();
-    if (jobNum)
+    QListViewItem* item = m_jobListView->selectedItem();
+    if (item)
     {
-        QStringList talkerCodesList = getTalkers();
-        // Create SelectTalker widget.
-        SelectTalkerWidget* stw = new SelectTalkerWidget();
-        stw->talkersList->setSelectionMode(QListView::Single);
-        // A list of the items in the listview.
-        QValueList<QListViewItem*> talkersListItems;
-        QListViewItem* talkerItem = 0;
-        // Fill rows and columns.
-        uint talkerCodesListCount = talkerCodesList.count();
-        for (uint index = 0; index < talkerCodesListCount; index++)
-        {
-            QString code = talkerCodesList[index];
-            TalkerCode parsedTalkerCode(code);
-            QString language = parsedTalkerCode.languageCode();
-            QString synthName = parsedTalkerCode.plugInName();
-            if (talkerItem)
-                talkerItem =
-                    new KListViewItem(stw->talkersList, talkerItem, language, synthName);
-            else
-                talkerItem =
-                    new KListViewItem(stw->talkersList, language, synthName);
-            updateTalkerItem(talkerItem, code);
-            talkersListItems.append(talkerItem);
-        }
-        // Display the listview in a dialog.
-        KDialogBase* dlg = new KDialogBase(
-            KDialogBase::Swallow,
-            i18n("Select Talker"),
-            KDialogBase::Help|KDialogBase::Ok|KDialogBase::Cancel,
-            KDialogBase::Cancel,
-            widget(),
-            "selectTalker_dlg",
-            true,
-            true);
-        dlg->setInitialSize(QSize(700, 300), false);
-        dlg->setMainWidget(stw);
-        // dlg->setHelp("configure-plugin", "kttsd");
-        if (dlg->exec() == QDialog::Accepted)
-        {
-            talkerItem = stw->talkersList->selectedItem();
-            if (talkerItem)
-            {
-                uint index = talkersListItems.findIndex(talkerItem);
-                changeTextTalker(talkerCodesList[index], jobNum);
-                refreshJob(jobNum);
-            }
-        }
-        delete stw;
-        delete dlg;
+        QString talkerID = item->text(jlvcTalkerID);
+        QStringList talkerIDs = m_talkerCodesToTalkerIDs.values();
+        int ndx = talkerIDs.findIndex(talkerID);
+        QString talkerCode;
+        if (ndx >= 0) talkerCode = m_talkerCodesToTalkerIDs.keys()[ndx];
+        SelectTalkerDlg dlg(widget(), "selecttalkerdialog", i18n("Select Talker"), talkerCode, true);
+        int dlgResult = dlg.exec();
+        if (dlgResult != KDialogBase::Accepted) return;
+        talkerCode = dlg.getSelectedTalkerCode();
+        int jobNum = item->text(jlvcJobNum).toInt();
+        changeTextTalker(talkerCode, jobNum);
+        refreshJob(jobNum);
     }
 }
 
@@ -607,17 +509,40 @@ void KttsJobMgrPart::slot_speak_clipboard()
     // Get the clipboard object.
     QClipboard *cb = kapp->clipboard();
 
+
     // Copy text from the clipboard.
-    QString text = cb->text();
+    QString text;
+    QMimeSource* data = cb->data();
+    if (data)
+    {
+        if (data->provides("text/html"))
+        {
+            if (supportsMarkup(NULL, KSpeech::mtHtml))
+            {
+                QByteArray d = data->encodedData("text/html");
+                text = QString(d);
+            }
+        }
+        if (data->provides("text/ssml"))
+        {
+            if (supportsMarkup(NULL, KSpeech::mtSsml))
+            {
+                QByteArray d = data->encodedData("text/ssml");
+                text = QString(d);
+            }
+        }
+    }
+    if (text.isEmpty())
+        text = cb->text();
 
     // Speak it.
-    if ( !text.isNull() ) 
+    if ( !text.isEmpty() )
     {
         uint jobNum = setText(text, NULL);
-        kdDebug() << "KttsJobMgrPart::slot_speak_clipboard: started jobNum " << jobNum << endl;
+        // kdDebug() << "KttsJobMgrPart::slot_speak_clipboard: started jobNum " << jobNum << endl;
         startText(jobNum);
         // Set flag so that the job we just created will be selected when textSet signal is received.
-        selectOnTextSet = true;        
+        m_selectOnTextSet = true;
     }
 }
 
@@ -737,7 +662,7 @@ void KttsJobMgrPart::refreshJob(uint jobNum)
     QListViewItem* item = findItemByJobNum(jobNum);
     if (item)
     {
-        item->setText(jlvcTalker, talkerID);
+        item->setText(jlvcTalkerID, talkerID);
         item->setText(jlvcState, stateToStr(state));
         item->setText(jlvcPosition, QString::number(seq));
         item->setText(jlvcSentences, QString::number(sentenceCount));
@@ -751,19 +676,19 @@ void KttsJobMgrPart::refreshJob(uint jobNum)
 */
 void KttsJobMgrPart::refreshJobListView()
 {
-    kdDebug() << "KttsJobMgrPart::refreshJobListView: Running" << endl;
+    // kdDebug() << "KttsJobMgrPart::refreshJobListView: Running" << endl;
     m_jobListView->clear();
     enableJobActions(false);
     enableJobPartActions(false);
     QString jobNumbers = getTextJobNumbers();
-    kdDebug() << "jobNumbers: " << jobNumbers << endl;
+    // kdDebug() << "jobNumbers: " << jobNumbers << endl;
     QStringList jobNums = QStringList::split(",", jobNumbers);
     QListViewItem* lastItem = 0;
     QStringList::ConstIterator endJobNums(jobNums.constEnd());
     for( QStringList::ConstIterator it = jobNums.constBegin(); it != endJobNums; ++it)
     {
         QString jobNumStr = *it;
-        kdDebug() << "jobNumStr: " << jobNumStr << endl;
+        // kdDebug() << "jobNumStr: " << jobNumStr << endl;
         uint jobNum = jobNumStr.toUInt(0, 10);
         QByteArray jobInfo = getTextJobInfo(jobNum);
         QDataStream stream(jobInfo, IO_ReadOnly);
@@ -796,13 +721,13 @@ void KttsJobMgrPart::refreshJobListView()
 
 /**
 * If nothing selected in Job List View and list not empty, select top item.
-* If nothing selected and list is empty, disable job buttons on toolbar.
+* If nothing selected and list is empty, disable job buttons.
 */
 void KttsJobMgrPart::autoSelectInJobListView()
 {
     // If something selected, nothing to do.
     if (m_jobListView->selectedItem()) return;
-    // If empty, disable job buttons on toolbar.
+    // If empty, disable job buttons.
     QListViewItem* item = m_jobListView->firstChild();
     if (!item)
     {
@@ -810,7 +735,7 @@ void KttsJobMgrPart::autoSelectInJobListView()
         enableJobPartActions(false);
     }
     else
-        // Select first item.  Should fire itemSelected event which will enable job buttons on toolbar.
+        // Select first item.  Should fire itemSelected event which will enable job buttons.
         m_jobListView->setSelected(item, true);
 }
 
@@ -834,67 +759,59 @@ QString KttsJobMgrPart::cachedTalkerCodeToTalkerID(const QString& talkerCode)
 }
 
 /**
- * Given an item in the talker listview and a talker code, sets the columns of the item.
- * @param talkerItem       QListViewItem.
- * @param talkerCode       Talker Code.
- */
-void KttsJobMgrPart::updateTalkerItem(QListViewItem* talkerItem, const QString &talkerCode)
-{
-    TalkerCode parsedTalkerCode(talkerCode);
-    QString fullLanguageCode = parsedTalkerCode.fullLanguageCode();
-    if (!fullLanguageCode.isEmpty())
-    {
-        QString language = parsedTalkerCode.languageCodeToLanguage(fullLanguageCode);
-        if (!language.isEmpty())
-        {
-            talkerItem->setText(tlvcLanguage, language);
-        }
-    }
-    // Don't update the Synthesizer name with plugInName.  The former is a translated
-    // name; the latter an English name.
-    // if (!plugInName.isEmpty()) talkerItem->setText(tlvcSynthName, plugInName);
-    if (!parsedTalkerCode.voice().isEmpty())
-        talkerItem->setText(tlvcVoice, parsedTalkerCode.voice());
-    if (!parsedTalkerCode.gender().isEmpty())
-        talkerItem->setText(tlvcGender, translatedGender(parsedTalkerCode.gender()));
-    if (!parsedTalkerCode.volume().isEmpty())
-        talkerItem->setText(tlvcVolume, translatedVolume(parsedTalkerCode.volume()));
-    if (!parsedTalkerCode.rate().isEmpty())
-        talkerItem->setText(tlvcRate, translatedRate(parsedTalkerCode.rate()));
-}
-
-/**
-* Enables or disables all the job-related buttons on the toolbar.
+* Enables or disables all the job-related buttons.
 * @param enable        True to enable the job-related butons.  False to disable.
 */
 void KttsJobMgrPart::enableJobActions(bool enable)
 {
-    for (uint index = 0; index < actionCollection()->count(); ++index)
+    if (!m_buttonBox) return;
+    QObjectList *l = m_buttonBox->queryList( "QPushButton", "job_*", true, true );
+    QObjectListIt it( *l ); // iterate over the buttons
+    QObject *obj;
+
+    while ( (obj = it.current()) != 0 ) {
+        // for each found object...
+        ++it;
+        ((QPushButton*)obj)->setEnabled( enable );
+    }
+    delete l; // delete the list, not the objects
+
+    if (enable)
     {
-        KAction* act = actionCollection()->action(index);
-        if (act)
+        // Later button only enables if currently selected list item is not bottom of list.
+        QListViewItem* item = m_jobListView->selectedItem();
+        if (item)
         {
-            QString actionName = act->name();
-            if (actionName.left(4) == "job_") act->setEnabled(enable);
+            bool enableLater = item->nextSibling();
+
+            l = m_buttonBox->queryList( "QPushButton", "job_later", false, true );
+            it = QObjectListIt( *l ); // iterate over the buttons
+            if ( (obj = it.current()) != 0 ) {
+                // for each found object...
+                ((QPushButton*)obj)->setEnabled( enableLater );
+            }
+            delete l; // delete the list, not the objects
         }
     }
 }
 
 /**
-* Enables or disables all the job part-related buttons on the toolbar.
+* Enables or disables all the job part-related buttons.
 * @param enable        True to enable the job par-related butons.  False to disable.
 */
 void KttsJobMgrPart::enableJobPartActions(bool enable)
 {
-    for (uint index = 0; index < actionCollection()->count(); ++index)
-    {
-        KAction* act = actionCollection()->action(index);
-        if (act)
-        {
-            QString actionName = act->name();
-            if (actionName.left(5) == "part_") act->setEnabled(enable);
-        }
+    if (!m_buttonBox) return;
+    QObjectList *l = m_buttonBox->queryList( "QPushButton", "part_*", true, true );
+    QObjectListIt it( *l ); // iterate over the buttons
+    QObject *obj;
+
+    while ( (obj = it.current()) != 0 ) {
+        // for each found object...
+        ++it;
+        ((QPushButton*)obj)->setEnabled( enable );
     }
+    delete l; // delete the list, not the objects
 }
 
 /** DCOP Methods connected to DCOP Signals emitted by KTTSD. */
@@ -902,7 +819,7 @@ void KttsJobMgrPart::enableJobPartActions(bool enable)
 /**
 * This signal is emitted when KTTSD starts or restarts after a call to reinit.
 */
-ASYNC KttsJobMgrPart::kttsdStarted() { slot_refresh(); };
+ASYNC KttsJobMgrPart::kttsdStarted() { slot_refresh(); }
 
 /**
 * This signal is emitted when the speech engine/plugin encounters a marker in the text.
@@ -923,7 +840,7 @@ ASYNC KttsJobMgrPart::markerSeen(const QCString&, const QString&)
  */
 ASYNC KttsJobMgrPart::sentenceStarted(const QCString&, const uint jobNum, const uint seq)
 {
-    kdDebug() << "KttsJobMgrPart::sentencedStarted: jobNum = " << jobNum << " seq = " << seq << endl;
+    // kdDebug() << "KttsJobMgrPart::sentencedStarted: jobNum = " << jobNum << " seq = " << seq << endl;
     QListViewItem* item = findItemByJobNum(jobNum);
     if (item)
     {
@@ -939,10 +856,10 @@ ASYNC KttsJobMgrPart::sentenceStarted(const QCString&, const uint jobNum, const
 * @param jobNum         Job number of the text job.
 * @param seq            Sequence number of the text.
 * @see getTextCount
-*/        
-ASYNC KttsJobMgrPart::sentenceFinished(const QCString&, const uint jobNum, const uint)
+*/
+ASYNC KttsJobMgrPart::sentenceFinished(const QCString& /*appId*/, const uint /*jobNum*/, const uint /*seq*/)
 {
-    kdDebug() << "KttsJobMgrPart::sentencedFinished: jobNum = " << jobNum << endl;
+    // kdDebug() << "KttsJobMgrPart::sentencedFinished: jobNum = " << jobNum << endl;
 /*
     QListViewItem* item = findItemByJobNum(jobNum);
     if (item)
@@ -981,10 +898,10 @@ ASYNC KttsJobMgrPart::textSet(const QCString&, const uint jobNum)
         stateToStr(state), QString::number(seq), QString::number(sentenceCount),
         QString::number(partNum), QString::number(partCount));
     // Should we select this job?
-    if (selectOnTextSet)
+    if (m_selectOnTextSet)
     {
         m_jobListView->setSelected(item, true);
-        selectOnTextSet = false;
+        m_selectOnTextSet = false;
     }
     // If a job not already selected, select this one.
     autoSelectInJobListView();
@@ -1028,7 +945,7 @@ ASYNC KttsJobMgrPart::textStarted(const QCString&, const uint jobNum)
 */
 ASYNC KttsJobMgrPart::textFinished(const QCString&, const uint jobNum)
 {
-    kdDebug() << "KttsJobMgrPart::textFinished: jobNum = " << jobNum << endl;
+    // kdDebug() << "KttsJobMgrPart::textFinished: jobNum = " << jobNum << endl;
     QListViewItem* item = findItemByJobNum(jobNum);
     if (item)
     {
@@ -1036,6 +953,7 @@ ASYNC KttsJobMgrPart::textFinished(const QCString&, const uint jobNum)
         // Update sentence pointer, since signal may not be emitted for final CR.
         refreshJob(jobNum);
     }
+    m_currentSentence->setText(QString::null);
 }
 
 /**
@@ -1045,7 +963,7 @@ ASYNC KttsJobMgrPart::textFinished(const QCString&, const uint jobNum)
 */
 ASYNC KttsJobMgrPart::textStopped(const QCString&, const uint jobNum)
 {
-    kdDebug() << "KttsJobMgrPart::textStopped: jobNum = " << jobNum << endl;
+    // kdDebug() << "KttsJobMgrPart::textStopped: jobNum = " << jobNum << endl;
     QListViewItem* item = findItemByJobNum(jobNum);
     if (item)
     {
@@ -1061,7 +979,7 @@ ASYNC KttsJobMgrPart::textStopped(const QCString&, const uint jobNum)
 */
 ASYNC KttsJobMgrPart::textPaused(const QCString&, const uint jobNum)
 {
-    kdDebug() << "KttsJobMgrPart::textPaused: jobNum = " << jobNum << endl;
+    // kdDebug() << "KttsJobMgrPart::textPaused: jobNum = " << jobNum << endl;
     QListViewItem* item = findItemByJobNum(jobNum);
     if (item)
     {
